/*
 * 쐬: 2004/01/01
 *
 */
package com.muimi.tree;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerFilter;
import org.eclipse.jface.viewers.ViewerSorter;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.program.Program;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;

public class ViewPart1 extends ViewPart {

	private TreeViewer treeViewer;


	public void createPartControl(Composite parent) {

		//c[r[A̍쐬
		treeViewer = new TreeViewer(parent);

		//RegvoC_̃Zbg
		treeViewer.setContentProvider(new TContentProvider());

		//xvoC_̃Zbg
		treeViewer.setLabelProvider(new TLabelProvider());

		//DoubleClickListener
		treeViewer.addDoubleClickListener(new DoubleClickListener());

		//SelectionProvider
		getSite().setSelectionProvider(treeViewer);

		//Filter
		treeViewer.addFilter(new TFilter());

		//Sorter
		treeViewer.setSorter(new TSorter());

		//Action낢
		addAction();

		//́I
		treeViewer.setInput(ResourcesPlugin.getWorkspace().getRoot());
	}


	public void setFocus() {
	}


	//ContentProvider̗
	class TContentProvider implements ITreeContentProvider {

		public Object[] getChildren(Object parent) {
			if (parent instanceof IContainer) {
				try {
					return ((IContainer) parent).members();
				} catch (CoreException e) {
					e.printStackTrace();
				}
			}

			return new Object[0];
		}

		public Object getParent(Object element) {
			return null;
		}

		public boolean hasChildren(Object element) {
			return getChildren(element).length > 0;
		}

		public Object[] getElements(Object input) {
			return getChildren(input);
		}

		public void dispose() {
		}

		public void inputChanged(
			Viewer viewer,
			Object oldInput,
			Object newInput) {
		}
	}


	//LabelProvider̗
	class TLabelProvider extends LabelProvider {

		public Image getImage(Object element) {
			if (element instanceof IContainer) {
				return PlatformUI.getWorkbench().getSharedImages().getImage(
					ISharedImages.IMG_OBJ_FOLDER);
			} else {
				return PlatformUI.getWorkbench().getSharedImages().getImage(
					ISharedImages.IMG_OBJ_FILE);
			}
		}

		public String getText(Object element) {
			return ((IResource) element).getName();
		}
	}


	//DoubleClickListener̗
	class DoubleClickListener implements IDoubleClickListener {

		public void doubleClick(DoubleClickEvent event) {
			IResource file =
				(IResource) ((StructuredSelection) event.getSelection())
					.getFirstElement();
			if (file instanceof IFile) {
				try {
					PlatformUI
						.getWorkbench()
						.getActiveWorkbenchWindow()
						.getActivePage()
						.openEditor(
						(IFile) file);
				} catch (PartInitException e) {
					e.printStackTrace();
				}
			}
		}
	}


	//\[^[̗
	class TSorter extends ViewerSorter{
		public int category(Object element) {
			return (element instanceof IContainer ? 0 : 1);
		}
	}
	
	//tB^[̗
	class TFilter extends ViewerFilter{
		public boolean select(Viewer viewer, Object parentElement, Object element) {
			return !((IResource)element).getName().startsWith(".");
		}
	}
	


	//낢ANVǉ
	void addAction() {
		IAction collapseAction = new CollapseAction();
		IAction expandAction = new ExpandAction();

		IActionBars actionBars = getViewSite().getActionBars();

		//j[
		IMenuManager menuBars = actionBars.getMenuManager();
		menuBars.add(collapseAction);
		menuBars.add(expandAction);

		//c[o[
		IToolBarManager toolBar = actionBars.getToolBarManager();
		toolBar.add(collapseAction);
		toolBar.add(expandAction);

		//|bvAbvj[
		MenuManager menuMgr = new MenuManager();
		menuMgr.setRemoveAllWhenShown(true);
		menuMgr.addMenuListener(new IMenuListener() {
			public void menuAboutToShow(IMenuManager menuManager) {
				menuManager.add(new RunAction());

				menuManager.add(new Separator());
				menuManager.add(
					new Separator(IWorkbenchActionConstants.MB_ADDITIONS));
			}
		});
		Menu menu = menuMgr.createContextMenu(treeViewer.getControl());
		treeViewer.getControl().setMenu(menu);
		getSite().registerContextMenu(menuMgr, treeViewer);
	}



	class ExpandAction extends Action {

		public ExpandAction(){
			setText("expand");
			setImageDescriptor(
				PlatformUI.getWorkbench().getSharedImages().getImageDescriptor(
				ISharedImages.IMG_TOOL_FORWARD));
		}

		public void run() {
			treeViewer.expandAll();
		}
	}

	class CollapseAction extends Action {

		public CollapseAction(){
			setText("collapse");
			setImageDescriptor(
				PlatformUI.getWorkbench().getSharedImages().getImageDescriptor(
				ISharedImages.IMG_TOOL_BACK));//AACR͓KłB
		}

		public void run() {
			treeViewer.collapseAll();
		}
	}
	
	class RunAction extends Action{
		public RunAction(){
			setText("run");
		}

		public void run() {
			IResource element = (IResource)((StructuredSelection)treeViewer.getSelection()).getFirstElement();
			if(element instanceof IFile){
				Program.launch(element.getLocation().toString());
			}
		}
	}

}
